/*QuadPlayer Song Optimizer
Copyright (c) 2005 James Byerly

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef QPSong__H
#define QPSong__H

#include <string>
#include <vector>
#include "TI83PlusProgram.h"

struct QPNote
{
	BYTE Left1, Left2, Right1, Right2;
	WORD Duration;
	bool operator ==(const QPNote& p_OtherNote) const
	{
		//compare all 5 elements
		return (Left1 == p_OtherNote.Left1)
			&& (Left2 == p_OtherNote.Left2)
			&& (Right1 == p_OtherNote.Right1)
			&& (Right2 == p_OtherNote.Right2)
			&& (Duration == p_OtherNote.Duration);
	}
	//use == to define !=
	bool operator !=(const QPNote& p_OtherNote) const
	{
		return !(*this == p_OtherNote);
	}
};

struct QPNoteCoord
{
	size_t Section;
	size_t Note;
	QPNoteCoord(){}
	QPNoteCoord(size_t p_Section, size_t p_Note):
		Section(p_Section), Note(p_Note){}
	bool operator <(const QPNoteCoord &p_OtherCoord) const
	{
		if(Section != p_OtherCoord.Section)
		{
			return Section < p_OtherCoord.Section;
		}
		return Note < p_OtherCoord.Note;
	}
};

struct QPSong
{
	std::string Signature;
	std::string Title;
	std::string Artist;
	std::string Album;

	TI83PlusProgram ProgramFile;
	
	typedef std::vector<int> SectionOrderVec;
	SectionOrderVec SectionOrder;//Stores index of section in SectionsNotes

	typedef std::vector<QPNote> SectionVec;
	typedef std::vector<SectionVec> SectionsNotesVec;
	SectionsNotesVec SectionsNotes;//first index is section, second is note

	//Load a TI83+ program file into this structure
	bool ReadFromFile(const char *p_FileName);
	//Save the TI83+ program structure in a file (MUST call ReadFromFile first and have it return true)
	bool WriteToFile(const char *p_FileName);
	//Expand the song into a single section
	void Expand(void);
};
#endif